/*
  This code reads the acceleration and gyroscope data from the MPU-9250 sensor and sends it via a nRF24L01+PA+LNA module to a receiver.
  To use these codes, the FaBo 202 9Axis MPU9250 library by Akira Sasaki released under the Apache license, version 2.0 should be included in the Arduino libraries.
  Also, to use the nRF24L01+PA+LNA module, the rf24 library from TMRh20 - Avamander released under the GNU general public license should be included in the Arduino libraries. A good tutorial for setting up the nRF24L01 module can be found at: https://howtomechatronics.com/tutorials/arduino/arduino-wireless-communication-nrf24l01-tutorial/
*/

  #include <SPI.h>
  #include <nRF24L01.h>
  #include <RF24.h>
 //#include <Wire.h>               
  #include <FaBo9Axis_MPU9250.h>


//nRF24L01+PA+LNA module to Arduino pin connection
  RF24 radio(7, 8); 
  
  //the adress/pipe through which the transmitter and receiver will communicate.
  const byte address[6] = "00001";
  
  //defining the data, which will be sent from the transmitter to the receiver
  struct Data_Package {   
    float ax,ay,az;
    float gx,gy,gz;
  };
  
  //creating a variable with the above structure
  Data_Package data; 

  FaBo9Axis fabo_9axis;

  //defining the IMU variables
  float ax,ay,az;   
  float gx,gy,gz;
  
  //Set offset. Note, the system could as well work without the offset
  boolean usingOffset = false; 
  float axOffset;
  float ayOffset;
  float azOffset;
  float gxOffset;
  float gyOffset;
  float gzOffset;

void setup() {

    // Serial to PC
    Serial.begin(115200);

    //setup needed for reading the IMU sensor
    if (fabo_9axis.begin()) {
      Serial.println("configured FaBo 9Axis I2C");
    } else {
      Serial.println("device error");
      while(1);
    }

    //checking weather the offset should be considered.
    if(usingOffset == true){    
           
      fabo_9axis.readAccelXYZ(&ax,&ay,&az);
      fabo_9axis.readGyroXYZ(&gx,&gy,&gz);
    
      axOffset = ax;
      ayOffset = ay;
      azOffset = az-1;
    
      gxOffset = gx;
      gyOffset = gy;
      gzOffset = gz;
    }

  //setup code for the rc transmitter - nRF24L01+PA+LNA
  radio.begin();
  radio.openWritingPipe(address);
  radio.setPALevel(RF24_PA_MIN);
  radio.stopListening();
}

void loop() {

  //reading the IMU data and writing the data in the ax, gx, ... variables
  fabo_9axis.readAccelXYZ(&ax,&ay,&az);
  fabo_9axis.readGyroXYZ(&gx,&gy,&gz);

  if(usingOffset == true){   
    ax = (ax-axOffset)*9.81;
    ay = (ay-ayOffset)*9.81;
    az = (az-azOffset)*9.81;
  
    gx = gx - gxOffset;
    gy = gy - gyOffset;
    gz = gz - gzOffset;
    
  } else {
    ax = ax*9.81;
    ay = ay*9.81;
    az = az*9.81;
  }

/*
  //For debugging: Printing the IMU data on the serial monitor
  Serial.println("ax, ay, and az:");
  Serial.print(ax);
  Serial.print(ay);
  Serial.println(az);
  Serial.println("gx, gy, and gz:");
  Serial.print(gx);
  Serial.print(gy);
  Serial.println(gz);
  delay(500);
*/

  //sending the data via the nRF24L01+ module at the control unit to nRF24L01+ module at the robot unit
  data.ax = ax;
  data.ay = ay;
  data.az = az;
  data.gx = gx;
  data.gy = gy;
  data.gz = gz;  

  //transmitting the struct via the nRF24L01+ module
  radio.write(&data, sizeof(data));
}


