/*
With this sketch, the robot can be remotely controlled using Android-based devices running ROS-Mobile.
*/

//Include libraries
#include <HardwareSerial.h>
#include <ODriveArduino.h>

//Set ODrive serial communication
ODriveArduino odrive(Serial2);

//Include ROS libraries
#include <ros.h>
#include <geometry_msgs/Twist.h>
#include <ros/time.h>
#include <tf/tf.h>
#include <tf/transform_broadcaster.h>
#include <nav_msgs/Odometry.h>
#include <std_msgs/String.h>
#include <geometry_msgs/Vector3Stamped.h>
#include <geometry_msgs/Vector3.h>

// Define the robot parameters
#define o2s_width (300) //mm
#define wheel_diameter (165.1) //mm 
#define wheel_circumference (518.362788)//mm
#define encoder_cpr (90) //count per revolution
#define rev (5) 

// ROS handle
ros::NodeHandle nh;
geometry_msgs::TransformStamped t;
nav_msgs::Odometry odom_msg;
ros::Publisher odom_pub("odom", &odom_msg);
tf::TransformBroadcaster broadcaster;

// command velocity variables
float request_x;
float request_z;

// set timers
unsigned long currentMillis;
long previousMillis = 0;

void velCallback( const geometry_msgs::Twist& vel)
{
request_x = vel.linear.x;
request_z = vel.angular.z;
}
// subscribe to the command velocity
ros::Subscriber<geometry_msgs::Twist> sub("cmd_vel" , velCallback);

//  Setup 

void setup() {

// ROS communication setup
nh.getHardware()->setBaud(115200);
nh.initNode(); 
nh.subscribe(sub); 
nh.advertise(odom_pub);
broadcaster.init(nh); 
Serial.begin(115200);
Serial2.begin(115200); 

//ODrive controller setup
int requested_state = AXIS_STATE_CLOSED_LOOP_CONTROL;
odrive.run_state(0, requested_state, false);
odrive.run_state(1, requested_state, false);

}

// Main loop

void loop() {

nh.spinOnce();

//run loop every 10 seconds
currentMillis = millis();
if (currentMillis - previousMillis >= 10) { 
previousMillis = currentMillis;

// scaling factor because the wheels are squashy / there is wheel slip etc.
//float modifier_linear = 1.08;
//float modifier_angular = 0.92;

// convert the parameter units
float conversion_linear = (rev * 1000)/wheel_circumference;
float conversion_angular = ((o2s_width/2) * rev)/wheel_circumference;
int m0_linear = request_x * conversion_linear; 
int m1_linear = request_x * conversion_linear; 
int m0_angular = request_z * conversion_angular;        
int m1_angular = request_z * conversion_angular;   

// reverse the direction of rotation of one of the wheel       
m1_linear = m1_linear*-1;

// set the velocity control
odrive.SetVelocity(0, m0_linear + m0_angular); 
odrive.SetVelocity(1, m1_linear + m1_angular);

  }
} 
